xquery version "3.1";
(:~  
 : REST-API Module to handle RESOURCES ("restapi", "http://dev.arokis.com/exist/term_expansion/resources_api")
 : *******************************************************************************************
 : This is the API module defining a REST-API to the texpan app. It is used to handle resources (add, remove, synchronise etc.)
 :  
 : Call REST-XQ with: localhost:8080/exist/restxq/via/...
 : @version 0.2 (2018-07-10)
 : @status develop
 : @author Uwe Sikora
 :)

module namespace restapi="http://dev.met.de/via/api";
import module namespace config="http://dev.met.de/via/config" at "xmldb:exist:///db/apps/via/modules/config.xqm";
import module namespace via="http://dev.met.de/via/main" at "xmldb:exist:///db/apps/via/modules/via.xqm";
import module namespace trivia="http://dev.met.de/via/trivia" at "xmldb:exist:///db/apps/via/modules/trivia.xqm";
import module namespace tgclient="https://sade.textgrid.de/ns/tgclient" at "xmldb:exist:///db/apps/via/modules/tgclient.xqm";
import module namespace console="http://exist-db.org/xquery/console";
import module namespace rest = "http://exquery.org/ns/restxq";
import module namespace transform="http://exist-db.org/xquery/transform";

declare namespace output = "http://www.w3.org/2010/xslt-xquery-serialization";
declare namespace test="http://exist-db.org/xquery/xqsuite";
declare namespace vias="http://dev.met.de/via/schema";
declare namespace http="http://expath.org/ns/http-client";
declare namespace request="http://exist-db.org/xquery/request";
declare namespace xlink="http://www.w3.org/1999/xlink";
(:############################# Modules Variables #############################:)

(:############################# Modules Functions #############################:)

(: let $login := xmldb:login("/db", "admin", "") :)


declare
    %rest:GET
    %rest:path("trivia/1.1/status")
    %output:method("xml")
function restapi:testing-this
    ( ) as item() {
    <trivia>This is Trivia! You shall comply!</trivia>
};



(: #########################################################################################
 : 
 :          TRIVIA API: TextGrid CRUD
 : 
 :########################################################################################## :)

(: ################### Trivia 1.0 API (OLD) ############# :)
(: This old api is kept by archival reasons and should not be considered as operative any more! :)
declare
    %rest:path("trivia/textgrid/crud/{$uri}/update")
    %rest:POST("{$data}")
    %rest:query-param("sid", "{$sid}", "")
    %rest:query-param("tei", "{$tei}", "false")
    %output:method("json")
function restapi:update-trivia-resource-in-tg
    ( $data as xs:string?, $uri, $sid , $tei ) as item()* {
    <rest:response>
        <http:response status="200">
          <http:header name="Content-Type" value="application/json"/>
          <http:header name="Access-Control-Allow-Origin" value="*"/>
          <http:header name="Access-Control-Allow-Methods" value="POST"/>
        </http:response>
    </rest:response>,
    let $decoded := util:base64-decode( $data )
    let $trivia-surface := parse-json( $decoded )
    
    let $trivia-xml := trivia:create-trivia-xml-resource-from-trivia-surface( $trivia-surface )
    (:let $tei-xml := trivia:create-tei-xml-resource( $trivia-surface ):)
    let $tg-config := map{ "tgcrudURL": $via:tgcrud }
    let $tg-config := if ( not($sid = "") ) then ( via:map-put($tg-config, 'sid', $sid) ) else ( $tg-config )
    let $update := trivia:update-resource-within-textgrid( $tg-config, $uri, $trivia-xml )
    return 
        map{"updated" : $uri, "responseData" : $update}
};


declare
    %rest:GET
    %rest:path("trivia/textgrid/crud/{$uri}/data")
    %rest:produces("application/json")
    %rest:form-param("sid", "{$sid}", "")
    %rest:form-param("convert", "{$convert}", "true")
    %output:method("json")
function restapi:textgrid-trivia-resource
    ( $uri as xs:string, $sid, $convert ) as item()* {
     <rest:response>
        <http:response status="200">
          <http:header name="Content-Language" value="en"/>
          <http:header name="Content-Type" value="application/json"/>
          <http:header name="Access-Control-Allow-Origin" value="*"/>
          <http:header name="Access-Control-Allow-Methods" value="GET, POST"/>
        </http:response>
    </rest:response>,
    let $trivia-resource := trivia:get-trivia-object-from-textgrid( $uri, $sid )
    return 
        if ( $convert = "true" ) then (
           
            if( $trivia-resource instance of map() ) then (
                trivia:convert-shapes-surface-to-via-surface($trivia-resource?object)
            ) else (
                map{ "error" : "not a trivia resource"}
            )
        ) else (
            $trivia-resource
        )
};


(: ########################## Trivia 1.1 API ########################### :)

declare
    %rest:GET
    %rest:path("trivia/1.1/textgrid/crud/{$uri}/data")
    %rest:produces("application/json")
    %rest:form-param("sid", "{$sid}", "")
    %rest:form-param("convert", "{$convert}", "true")
    %output:method("json")
function restapi:textgrid-trivia-resource-with-tei
    ( $uri as xs:string, $sid, $convert ) as item()* {
     <rest:response>
        <http:response status="200">
          <http:header name="Content-Language" value="en"/>
          <http:header name="Content-Type" value="application/json"/>
          <http:header name="Access-Control-Allow-Origin" value="*"/>
          <http:header name="Access-Control-Allow-Methods" value="GET, POST"/>
        </http:response>
    </rest:response>,
    let $trivia-object := trivia:build-trivia-object-from-textgrid-resources( $uri, $sid )
    return 
        if ( $convert = "true" ) then (
            $trivia-object?object
        ) else ()
};


declare
    %rest:path("trivia/1.1/textgrid/crud/{$uri}/update")
    %rest:POST("{$data}")
    %rest:query-param("sid", "{$sid}", "")
    %rest:query-param("tei", "{$tei}", "false")
    %rest:consumes("application/json")
    %output:method("json")
function restapi:update-trivia-resource-in-tg-1.1
    ( $data as xs:string?, $uri, $sid , $tei ) as item()* {
    <rest:response>
        <http:response status="200">
          <http:header name="Content-Type" value="application/json"/>
          <http:header name="Access-Control-Allow-Origin" value="*"/>
          <http:header name="Access-Control-Allow-Methods" value="POST"/>
        </http:response>
    </rest:response>,
    let $decoded := util:base64-decode( $data )
    let $trivia-surface := parse-json( $decoded )
    let $tei-uri := trivia:get-referenced-text-resource-uri-from-trivia-surface( $trivia-surface )
    
    let $tg-config := map{ "tgcrudURL": $via:tgcrud }
    let $tg-config := if ( not($sid = "") ) then ( via:map-put($tg-config, 'sid', $sid) ) else ( $tg-config )
    return (
        (: create trivia resource :)
        let $trivia-xml := trivia:create-trivia-xml-resource-from-trivia-surface( $trivia-surface )
        let $update-trivia-resource := trivia:update-resource-within-textgrid( $tg-config, $uri, $trivia-xml )
        let $update-tei-resource := if ( $tei-uri ) then (
                let $tei := trivia:get-referenced-text-resource-from-trivia-surface-via-textgrid( $trivia-surface, $via:tg-config("sid") )
                let $updated-tei := trivia:update-tei-xml-resource-from-trivia-surface( $tei, $trivia-surface )
                return 
                    trivia:update-resource-within-textgrid( $tg-config, $tei-uri, $updated-tei )
            ) else ()
        return 
            map{"updated" : array{$uri, $tei-uri}, "responseData" : array{$update-trivia-resource, $update-tei-resource} }
    )
    (:
    let $trivia-xml := trivia:create-trivia-xml-resource-from-trivia-surface( $trivia-surface )
    (:let $tei-xml := trivia:create-tei-xml-resource( $trivia-surface ):)
    let $tg-config := map{ "tgcrudURL": $via:tgcrud }
    let $tg-config := if ( not($sid = "") ) then ( via:map-put($tg-config, 'sid', $sid) ) else ( $tg-config )
    let $update := trivia:update-trivia-xml-within-textgrid( $tg-config, $uri, $trivia-xml )
    return 
        map{"updated" : $uri, "responseData" : $update}
    :)
};


declare
    %rest:path("trivia/1.2/textgrid/crud/{$uri}/update")
    %rest:POST("{$data}")
    %rest:query-param("sid", "{$sid}", "")
    %rest:query-param("tei", "{$tei}", "false")
    %rest:consumes("application/json")
    %output:method("json")
function restapi:update-trivia-resource-in-tg-1.2
    ( $data as xs:string?, $uri, $sid , $tei ) as item()* {
    <rest:response>
        <http:response status="200">
          <http:header name="Content-Type" value="application/json"/>
          <http:header name="Access-Control-Allow-Origin" value="*"/>
          <http:header name="Access-Control-Allow-Methods" value="POST"/>
        </http:response>
    </rest:response>,
    let $decoded := util:base64-decode( $data )
    let $trivia-surface := parse-json( $decoded )
    (:let $tei-uri := trivia:get-referenced-text-resource-uri-from-trivia-surface( $trivia-surface ):)
    
    let $tg-config := map{ "tgcrudURL": $via:tgcrud }
    let $tg-config := if ( not($sid = "") ) then ( via:map-put($tg-config, 'sid', $sid) ) else ( $tg-config )
    
    let $metadata := trivia:get-trivia-surface-metadata( $trivia-surface )
    let $textRef := $trivia-surface?textRef
    let $imageRef := $trivia-surface?imageRef
    let $file-attributes := $metadata?file_attributes
    let $zones-xml := trivia:create-zones-xml-from-trivia-zones( $trivia-surface?zones )
    let $svg-xml := element { QName("http://www.w3.org/2000/svg", "svg:svg") }{
        element { QName("http://www.w3.org/2000/svg", "svg:g") }{
            element { QName("http://www.w3.org/2000/svg", "svg:image") }{
                attribute { "height" }{ $file-attributes?naturalHeight },
                attribute { "width" }{ $file-attributes?naturalWidth },
                attribute { "xlink:href" }{ $file-attributes?imageRef }
            },
            trivia:create-svg-shapes-xml-from-trivia-zones( $trivia-surface?zones )
        }
    }
    return (
        $zones-xml, $svg-xml
    )
};


declare
    %rest:path("trivia/1.1/textgrid/crud/create")
    %rest:GET
    %rest:form-param("sid", "{$sid}", "")
    %rest:form-param("pid", "{$pid}", "")
    %rest:form-param("imageUri", "{$imageUri}", "")
    %rest:form-param("textUri", "{$textUri}", "")
    %rest:form-param("title", "{$title}", "new trivia resource")
    %rest:form-param("aggregation", "{$aggregation}", "")
    %output:method("json")
function restapi:create-trivia-resource-in-tg
    ( $data as xs:string?, $sid, $pid, $title, $imageUri, $textUri, $aggregation ) as item()* {
    <rest:response>
        <http:response status="200">
          <http:header name="Content-Type" value="application/json"/>
          <http:header name="Access-Control-Allow-Origin" value="*"/>
          <http:header name="Access-Control-Allow-Methods" value="POST"/>
        </http:response>
    </rest:response>,
    
    if ( not($imageUri = '') ) then (
        
        let $tg-config := map{
            "sid" : $sid,
            "pid" : $pid,
            "tgcrudURL" : $via:tg-config("tgcrudURL")
        }
        
        let $create-tei-resource := if ( $textUri = '' ) then ( trivia:create-tei-xml-resource-in-textgrid( $tg-config, $title||'_tei' )?uri ) else ( $textUri )
        let $create-trivia-resource := trivia:create-trivia-xml-resource-in-textgrid( $tg-config, $title, $imageUri, $create-tei-resource )
        
        return
            (:map {
              "aggregation": "textgrid:3r345.0",
              "created": map {
                "teiResource": map {
                  "uri": "textgrid:3r3w2",
                  "title": "a new trivia resource_tei"
                },
                "triviaResource": map {
                  "uri": "textgrid:3r3w3",
                  "title": "a new trivia resource"
                }
              }
            }:)
            
            if ( not($aggregation = '') ) then (
                let $push := tgclient:push-to-aggregation( $tg-config, $create-trivia-resource?uri, $aggregation )
                let $push := if ( count($create-trivia-resource) > 0 ) then ( tgclient:push-to-aggregation( $tg-config, $create-tei-resource, $aggregation ) ) else () 
                return 
                    map{ 
                        "created" : map{
                            "triviaResource" : $create-trivia-resource,
                            "teiResource" : $create-tei-resource 
                        }, 
                        "aggregation" : $aggregation
                    }
            ) else (
                map{ 
                    "created" : map{
                        "triviaResource" : $create-trivia-resource,
                        "teiResource" : $create-tei-resource 
                    }
                }
            )
    
    ) else (
        map{ 
            "error" : 'no image id given'
        }
    )
    
};



(: #########################################################################################
 : 
 :          TRIVIA API: Authenticate via Backend
 : 
 :########################################################################################## :)

declare
    %rest:GET
    %rest:path("trivia/1.1/auth")
    %rest:produces("application/json")
    (:%output:media-type("application/json"):)
    %output:method("json")
function restapi:get-credentials
    () as item()* {
    <rest:response>
        <http:response status="200">
          <http:header name="Content-Language" value="en"/>
          <http:header name="Content-Type" value="application/json"/>
          <http:header name="Access-Control-Allow-Origin" value="*"/>
          <http:header name="Access-Control-Allow-Methods" value="GET, POST"/>
        </http:response>
    </rest:response>,
    let $config := $config:data-root||"/json/config.json"
    let $data := parse-json(util:binary-to-string(  util:binary-doc($config) ))
    return $data
};